@@ -20,7 +20,7 @@ def configuration(parent_package='', top_path=None):
                          **numpy_nodepr_api)
 
     config.add_extension('_spectral', sources=['_spectral.c'])
-    config.add_extension('_max_len_seq', sources=['_max_len_seq.c'])
+    config.add_extension('_max_len_seq_inner', sources=['_max_len_seq_inner.c'])
 
     spline_src = ['splinemodule.c', 'S_bspline_util.c', 'D_bspline_util.c',
                   'C_bspline_util.c', 'Z_bspline_util.c', 'bspline_util.c']
